@@ -3,6 +3,8 @@
 use cortex_m::asm;
 use embedded_hal::adc::{Channel, OneShot};
 
+use core::ops::Deref;
+
 use crate::{
     pac::{self, RCC},
     rcc_en_reset,
@@ -12,6 +14,14 @@ use crate::{
 use cfg_if::cfg_if;
 use paste::paste;
 
+#[cfg(feature = "g0")]
+use crate::pac::dma as dma_p;
+#[cfg(any(feature = "f3", feature = "l4", feature = "g4"))]
+use crate::pac::dma1 as dma_p;
+
+#[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
+use crate::dma::{self, ChannelCfg, Dma, DmaChannel, DmaInput};
+
 const MAX_ADVREGEN_STARTUP_US: u32 = 10;
 
 /// https://github.com/rust-embedded/embedded-hal/issues/267
@@ -40,21 +50,39 @@ pub mod AdcChannel {
     pub struct C19;
     pub struct C20;
 }
-//
-// #[derive(Clone, Copy)]
-// enum AdcNum {
-//     One,
-//     Two,
-//     Three,
-//     Four,
-//     #[cfg(feature = "g4")]
-//     Five,
-// }
+
+#[derive(Clone, Copy)]
+pub enum AdcDevice {
+    One,
+    Two,
+    Three,
+    Four,
+    #[cfg(feature = "g4")]
+    Five,
+}
+
+#[derive(Copy, Clone)]
+#[repr(u8)]
+/// See L44 RM, section 16.5: ADC interrupts
+pub enum AdcInterrupt {
+    Ready,
+    EndOfConversion,
+    EnfOfSequence,
+    EndofConversionInjected,
+    EndOfSequenceInjected,
+    Watchdog1,
+    Watchdog2,
+    Watchdog3,
+    EndOfSamplingPhase,
+    Overrun,
+    InjectedOverflow,
+}
 
 /// Analog to Digital converter peripheral
 pub struct Adc<ADC> {
     /// ADC Register
     regs: ADC,
+    device: AdcDevice,
     ckmode: ClockMode,
     operation_mode: OperationMode,
     // Most families use u8 values for calibration, but H7 uses u16.
@@ -116,33 +144,30 @@ pub enum OperationMode {
     Continuous = 1,
 }
 
-// #[cfg(any(feature = "l4", feature = "l5"))]
-// #[derive(Clone, Copy, PartialEq)]
-// #[repr(u8)]
-// /// ADC Clock mode
-// pub enum ClockMode {
-//     // todo
-//     Sysclock = 0b00,
-//     PllSai1 = 0b01,
-// }
-//
-// #[cfg(any(feature = "l4", feature = "l5"))]
-// impl Default for ClockMode {
-//     fn default() -> Self {
-//         Self::Sysclock
-//     }
-// }
+// todo: Check the diff ways of configuring clock; i don't think teh enum below covers all.(?)
 
-// todo: Clock mode for other MCUs!
-
-// #[cfg(any(feature = "f3"))]
 #[derive(Clone, Copy, PartialEq)]
 #[repr(u8)]
 /// ADC Clock mode
+/// (L44 RM, Section 16.4.3) The input clock is the same for the three ADCs and can be selected between two different
+/// clock sources (see Figure 40: ADC clock scheme):
+/// 1. The ADC clock can be a specific clock source. It can be derived from the following
+/// clock sources:
+/// – The system clock
+/// – PLLSAI1 (single ADC implementation)
+/// Refer to RCC Section for more information on how to generate ADC dedicated clock.
+/// To select this scheme, bits CKMODE[1:0] of the ADCx_CCR register must be reset.
+/// 2. The ADC clock can be derived from the AHB clock of the ADC bus interface, divided by
+/// a programmable factor (1, 2 or 4). In this mode, a programmable divider factor can be
+/// selected (/1, 2 or 4 according to bits CKMODE[1:0]).
+/// To select this scheme, bits CKMODE[1:0] of the ADCx_CCR register must be different
+/// from “00”.
 pub enum ClockMode {
     // Use Kernel Clock adc_ker_ck_input divided by PRESC. Asynchronous to AHB clock
-    // ASYNC = 0b00,
-    /// Use AHB clock rcc_hclk3. In this case rcc_hclk must equal sys_d1cpre_ck
+    Async = 0b00,
+    /// Use AHB clock rcc_hclk3.
+    /// "For option 2), a prescaling factor of 1 (CKMODE[1:0]=01) can be used only if the AHB
+    /// prescaler is set to 1 (HPRE[3:0] = 0xxx in RCC_CFGR register)."
     SyncDiv1 = 0b01,
     /// Use AHB clock rcc_hclk3 divided by 2
     SyncDiv2 = 0b10,
@@ -150,7 +175,6 @@ pub enum ClockMode {
     SyncDiv4 = 0b11,
 }
 
-// #[cfg(any(feature = "f3"))]
 impl Default for ClockMode {
     fn default() -> Self {
         Self::SyncDiv2
@@ -221,49 +245,49 @@ macro_rules! hal {
                 ///
                 pub fn [<new_ $adc>]<C: ClockCfg>(
                     regs: pac::$ADC,
+                    device: AdcDevice,
                     adc_common : &mut pac::$ADC_COMMON,
                     ckmode: ClockMode,
                     clocks: &C,
                     rcc: &mut RCC,
                 ) -> Self {
-                    let mut this_adc = Self {
+                    let mut result = Self {
                         regs,
+                        device,
                         ckmode,
                         operation_mode: OperationMode::OneShot,
                         cal_single_ended: None,
                         cal_differential: None,
                     };
 
-                    this_adc.enable_clock(adc_common, rcc);
-                    this_adc.set_align(Align::default());
+                    result.enable_clock(adc_common, rcc);
+                    result.set_align(Align::default());
 
-                    this_adc.advregen_enable(clocks);
+                    result.advregen_enable(clocks);
 
-                    this_adc.calibrate(InputType::SingleEnded, clocks);
-                    this_adc.calibrate(InputType::Differential, clocks);
+                    result.calibrate(InputType::SingleEnded, clocks);
+                    result.calibrate(InputType::Differential, clocks);
 
                     // Reference Manual: "ADEN bit cannot be set during ADCAL=1
                     // and 4 ADC clock cycle after the ADCAL
                     // bit is cleared by hardware."
-                    let adc_per_cpu_cycles = match this_adc.ckmode {
+                    let adc_per_cpu_cycles = match result.ckmode {
+                        ClockMode::Async => unimplemented!(),
                         ClockMode::SyncDiv1 => 1,
                         ClockMode::SyncDiv2 => 2,
                         ClockMode::SyncDiv4 => 4,
                     };
                     asm::delay(adc_per_cpu_cycles * 4);
-                    this_adc.enable();
+                    result.enable();
 
-                    this_adc.setup_oneshot(); // todo: Setup Continuous
+                    result.setup_oneshot(); // todo: Setup Continuous
 
-                    this_adc
+                    result
                 }
             }
 
             /// Enable the ADC clock, and set the clock mode.
             fn enable_clock(&self, common_regs: &mut pac::$ADC_COMMON, rcc: &mut RCC) {
-                 // `common_regs` is the same as `self.regs` for non-f3. On f3, it's a diff block,
-                 // eg `adc12`.
-                 // todo: Macros instead of AdcNum? More consistent with other modules. Not sure which is more apt.
                 paste! {
                     cfg_if! {
                         if #[cfg(any(feature = "f3", feature = "h7"))] {
@@ -275,19 +299,12 @@ macro_rules! hal {
                             rcc_en_reset!(ahb1, [<adc $rcc_num>], rcc);
                         } else if #[cfg(any(feature = "g4"))] {
                             rcc_en_reset!(ahb2, [<adc $rcc_num>], rcc);
-                        } else {  // ie L4, L5 etc.
+                        } else {  // ie L4, L5, G0(?)
                             rcc_en_reset!(ahb2, adc, rcc);
                         }
                     }
                 }
-
-                cfg_if! {
-                    if #[cfg(any(feature = "g4", feature = "h7"))] {  // todo why can't we find ccr?
-                        common_regs.ccr.modify(|_, w| unsafe { w.ckmode().bits(self.ckmode as u8) });
-                    } else {
-                        common_regs.ccr.modify(|_, w| unsafe { w.ckmode().bits(self.ckmode as u8) });
-                    }
-                }
+                common_regs.ccr.modify(|_, w| unsafe { w.ckmode().bits(self.ckmode as u8) });
 
             }
 
@@ -298,6 +315,13 @@ macro_rules! hal {
 
                 self.regs.cfgr.modify(|_, w| w
                     .cont().clear_bit()  // single conversion mode.
+                    // RM:
+                    // • OVRMOD=0: The overrun event preserves the data register from being overrun: the
+                    // old data is maintained and the new conversion is discarded and lost. If OVR remains at
+                    // 1, any further conversions will occur but the result data will be also discarded.
+                    // • OVRMOD=1: The data register is overwritten with the last conversion result and the
+                    // previous unread data is lost. If OVR remains at 1, any further conversions will operate
+                    // normally and the ADC_DR register will always contain the latest converted data.
                     .ovrmod().clear_bit()  // preserve DR data
                 );
 
@@ -311,9 +335,13 @@ macro_rules! hal {
                     panic!("ADC sequence length must be in 1..=16")
                 }
 
-                // typo
+                // todo: Swap these once the next version of the PACs are out.
+                // self.regs.sqr1.modify(|_, w| unsafe { w.l().bits(len - 1) });
                 cfg_if! {
-                    if #[cfg(any(feature = "l4x1", feature = "l4x2", feature = "l4x3", feature = "l4x5"))] {
+                    // todo: We've put l4x2 on the newer one, since we're using the workaround local
+                    // todo PAC, which has the newer approach.
+                    // if #[cfg(any(feature = "l4x1", feature = "l4x2", feature = "l4x3", feature = "l4x5"))] {
+                    if #[cfg(any(feature = "l4x1", feature = "l4x3", feature = "l4x5"))] {
                         self.regs.sqr1.modify(|_, w| unsafe { w.l3().bits(len - 1) });
                     } else {
                         self.regs.sqr1.modify(|_, w| unsafe { w.l().bits(len - 1) });
@@ -334,7 +362,7 @@ macro_rules! hal {
             /// operation.
             fn enable(&mut self) {
                 // 1. Clear the ADRDY bit in the ADC_ISR register by writing ‘1’.
-                self.regs.isr.modify(|_, w| w.adrdy().clear_bit());
+                self.regs.isr.modify(|_, w| w.adrdy().set_bit());
                 // 2. Set ADEN=1.
                 self.regs.cr.modify(|_, w| w.aden().set_bit());  // Enable
                 // 3. Wait until ADRDY=1 (ADRDY is set after the ADC startup time). This can be done
@@ -379,9 +407,10 @@ macro_rules! hal {
                     self.regs.cr.modify(|_, w| {
                         w.adstp().set_bit();
                         w.jadstp().set_bit()
-                    })
+                    });
+
+                    while self.regs.cr.read().adstart().bit_is_set() || self.regs.cr.read().jadstart().bit_is_set() {}
                 }
-                while self.regs.cr.read().adstart().bit_is_set() || self.regs.cr.read().jadstart().bit_is_set() {}
             }
 
             /// Check if the ADC is enabled.
@@ -562,49 +591,40 @@ macro_rules! hal {
                     self.disable();
                 }
 
-                // todo: Figure out how you do the bit shift math here; we don't have individual fields.
-                // difsel!(self.regs, channel, input_type);
-                // self.regs.difsel.write(|w| w.difsel_1_15r().bits(input_type as u8));
+                // Note that we don't use the `difsel` PAC accessor here; not required,
+                // and would require some feature gating due to differences in field names among some
+                // variants. Would also requires shifting `channel` by 1 more.
+                let val = self.regs.difsel.read().bits();
+                self.regs.difsel.write(|w| unsafe { w.bits(
+                    val | ((input_type as u32) << channel)
+                )});
 
                 if was_enabled {
                     self.enable();
                 }
             }
 
-            /// Start a conversion: Either a single measurement, or continuous conversions.
-            /// See L4 RM 16.4.15 for details.
-            pub fn start_conversion(&mut self, chan: u8, mode: OperationMode) {
-                // Set continuous or differential mode.
-                self.regs.cfgr.modify(|_, w| w.cont().bit(mode as u8 != 0));
-                self.select_channel(chan);
-
-                // L4 RM: In Single conversion mode, the ADC performs once all the conversions of the channels.
-                // This mode is started with the CONT bit at 0 by either:
-                // • Setting the ADSTART bit in the ADC_CR register (for a regular channel)
-                // • Setting the JADSTART bit in the ADC_CR register (for an injected channel)
-                // • External hardware trigger event (for a regular or injected channel)
-                // (Here, we assume a regular channel)
-                self.regs.cr.modify(|_, w| w.adstart().set_bit());  // Start
-
-                // After the regular sequence is complete, after each conversion is complete,
-                // the EOC (end of regular conversion) flag is set.
-                // After the regular sequence is complete: The EOS (end of regular sequence) flag is set.
-                // (We're ignoring eoc, since this module doesn't currently support sequences)
-                while self.regs.isr.read().eos().bit_is_clear() {}  // wait until complete.
-            }
-
-            /// Read data from a conversion. In OneShot mode, this will generally be run right
-            /// after `start_conversion`.
-            pub fn read_result(&mut self) -> u16 {
-                self.regs.dr.read().bits() as u16
-            }
-
-            /// Select the channel to sample. Note that this register allows setting a sequence,
-            /// but for now, we only support converting one channel at a time.
-            /// (Single-ended or differential, but not a sequence.)
-            fn select_channel(&self, chan: u8) {
-                // Channel as u8 is the ADC channel to use.
-                self.regs.sqr1.modify(|_, w| unsafe { w.sq1().bits(chan) });
+            /// Select a sequence to sample, by inputting a single channel and position.
+            pub fn set_sequence(&mut self, chan: u8, position: u8) {
+                match position {
+                    1 => self.regs.sqr1.modify(|_, w| unsafe { w.sq1().bits(chan) }),
+                    2 => self.regs.sqr1.modify(|_, w| unsafe { w.sq2().bits(chan) }),
+                    3 => self.regs.sqr1.modify(|_, w| unsafe { w.sq3().bits(chan) }),
+                    4 => self.regs.sqr1.modify(|_, w| unsafe { w.sq4().bits(chan) }),
+                    5 => self.regs.sqr2.modify(|_, w| unsafe { w.sq5().bits(chan) }),
+                    6 => self.regs.sqr2.modify(|_, w| unsafe { w.sq6().bits(chan) }),
+                    7 => self.regs.sqr2.modify(|_, w| unsafe { w.sq7().bits(chan) }),
+                    8 => self.regs.sqr2.modify(|_, w| unsafe { w.sq8().bits(chan) }),
+                    9 => self.regs.sqr2.modify(|_, w| unsafe { w.sq9().bits(chan) }),
+                    10 => self.regs.sqr3.modify(|_, w| unsafe { w.sq10().bits(chan) }),
+                    11 => self.regs.sqr3.modify(|_, w| unsafe { w.sq11().bits(chan) }),
+                    12 => self.regs.sqr3.modify(|_, w| unsafe { w.sq12().bits(chan) }),
+                    13 => self.regs.sqr3.modify(|_, w| unsafe { w.sq13().bits(chan) }),
+                    14 => self.regs.sqr3.modify(|_, w| unsafe { w.sq14().bits(chan) }),
+                    15 => self.regs.sqr4.modify(|_, w| unsafe { w.sq15().bits(chan) }),
+                    16 => self.regs.sqr4.modify(|_, w| unsafe { w.sq16().bits(chan) }),
+                    _ => panic!("Sequence out of bounds. Only 16 positions are available."),
+                }
             }
 
             /// Select the sample time for a given channel.
@@ -639,11 +659,179 @@ macro_rules! hal {
                 }
             }
 
+            /// Start a conversion: Either a single measurement, or continuous conversions.
+            /// See L4 RM 16.4.15 for details.
+            pub fn start_conversion(&mut self, sequence: &[u8], mode: OperationMode) {
+                // Set continuous or differential mode.
+                self.regs.cfgr.modify(|_, w| w.cont().bit(mode as u8 != 0));
+                // todo: You should call this elsewhere, once, to prevent unneded reg writes.
+                for (i, channel) in sequence.iter().enumerate() {
+                    self.set_sequence(*channel, i as u8 + 1); // + 1, since sequences start at 1.
+                }
+
+                // L4 RM: In Single conversion mode, the ADC performs once all the conversions of the channels.
+                // This mode is started with the CONT bit at 0 by either:
+                // • Setting the ADSTART bit in the ADC_CR register (for a regular channel)
+                // • Setting the JADSTART bit in the ADC_CR register (for an injected channel)
+                // • External hardware trigger event (for a regular or injected channel)
+                // (Here, we assume a regular channel)
+                self.regs.cr.modify(|_, w| w.adstart().set_bit());  // Start
+
+                // After the regular sequence is complete, after each conversion is complete,
+                // the EOC (end of regular conversion) flag is set.
+                // After the regular sequence is complete: The EOS (end of regular sequence) flag is set.
+                // (We're ignoring eoc, since this module doesn't currently support sequences)
+                while self.regs.isr.read().eos().bit_is_clear() {}  // wait until complete.
+            }
+
+            /// Read data from a conversion. In OneShot mode, this will generally be run right
+            /// after `start_conversion`.
+            pub fn read_result(&mut self) -> u16 {
+                // todo: read_volatile to be more efficient due to only reading 16 bits?
+                self.regs.dr.read().bits() as u16
+            }
+
             /// Take a single reading, in OneShot mode
             pub fn read(&mut self, channel: u8) -> u16 {
-                self.start_conversion(channel, OperationMode::OneShot);
+                self.start_conversion(&[channel], OperationMode::OneShot);
                 self.read_result()
             }
+
+            #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+            /// Take a one shot reading, using DMA. See L44 RM, 16.4.27: "DMA one shot mode".
+            /// Note that the `channel` argument is only used on F3 and L4.
+            pub fn read_dma<D>(&mut self, buf: &mut [u16], adc_channel: u8, dma_channel: DmaChannel, dma: &mut Dma<D>)
+            where
+                D: Deref<Target = dma_p::RegisterBlock>,
+            {
+
+                let (ptr, len) = (buf.as_mut_ptr(), buf.len());
+                // The software is allowed to write (dmaen and dmacfg) only when ADSTART=0 and JADSTART=0 (which
+                // ensures that no conversion is ongoing)
+                // Todo: Should these settings be handled in `init`?
+                self.stop_conversions();
+                self.regs.cfgr.modify(|_, w| {
+                    w.dmacfg().clear_bit(); // (one shot mode)  todo: Circular mode option; dmacfg=1.
+                    w.dmaen().set_bit()
+                });
+
+                // L44 RM, Table 41. "DMA1 requests for each channel
+                // todo: DMA2 support.
+                #[cfg(any(feature = "f3", feature = "l4"))]
+                let channel = match self.device {
+                    AdcDevice::One => DmaInput::Adc1.dma1_channel(),
+                    AdcDevice::Two => DmaInput::Adc2.dma1_channel(),
+                    _ => panic!("DMA on ADC beyond 2 is not supported. If it is for your MCU, please submit an issue \
+                or PR on Github.")
+                };
+
+                #[cfg(feature = "l4")]
+                match self.device {
+                    AdcDevice::One => dma.channel_select(DmaInput::Adc1),
+                    AdcDevice::Two => dma.channel_select(DmaInput::Adc2),
+                    _ => unimplemented!(),
+                }
+
+                self.set_sequence(adc_channel, 1);
+                // todo: Support sequences.
+
+                self.regs.cr.modify(|_, w| w.adstart().set_bit());  // Start
+
+                // Since converted channel values are stored into a unique data register, it is useful to use
+                // DMA for conversion of more than one channel. This avoids the loss of the data already
+                // stored in the ADC_DR register.
+                // When the DMA mode is enabled (DMAEN bit set to 1 in the ADC_CFGR register in single
+                // ADC mode or MDMA different from 0b00 in dual ADC mode), a DMA request is generated
+                // after each conversion of a channel. This allows the transfer of the converted data from the
+                // ADC_DR register to the destination location selected by the software.
+                // Despite this, if an overrun occurs (OVR=1) because the DMA could not serve the DMA
+                // transfer request in time, the ADC stops generating DMA requests and the data
+                // corresponding to the new conversion is not transferred by the DMA. Which means that all
+                // the data transferred to the RAM can be considered as valid.
+                // Depending on the configuration of OVRMOD bit, the data is either preserved or overwritten
+                // (refer to Section : ADC overrun (OVR, OVRMOD)).
+                // The DMA transfer requests are blocked until the software clears the OVR bit.
+                // Two different DMA modes are proposed depending on the application use and are
+                // configured with bit DMACFG of the ADC_CFGR register in single ADC mode, or with bit
+                // DMACFG of the ADC_CCR register in dual ADC mode:
+                // • DMA one shot mode (DMACFG=0).
+                // This mode is suitable when the DMA is programmed to transfer a fixed number of data.
+                // • DMA circular mode (DMACFG=1)
+                // This mode is suitable when programming the DMA in circular mode.
+
+
+                // In [DMA one shot mode], the ADC generates a DMA transfer request each time a new conversion data
+                // is available and stops generating DMA requests once the DMA has reached the last DMA
+                // transfer (when DMA_EOT interrupt occurs - refer to DMA paragraph) even if a conversion
+                // has been started again.
+                // When the DMA transfer is complete (all the transfers configured in the DMA controller have
+                // been done):
+                // • The content of the ADC data register is frozen.
+                // • Any ongoing conversion is aborted with partial result discarded.
+                // • No new DMA request is issued to the DMA controller. This avoids generating an
+                // overrun error if there are still conversions which are started.
+                // • Scan sequence is stopped and reset.
+                // • The DMA is stopped.
+
+                dma.cfg_channel(
+                    dma_channel,
+                    &self.regs.dr as *const _ as u32,
+                    ptr as u32,
+                    len as u16,
+                    dma::Direction::ReadFromPeriph,
+                    dma::DataSize::S16,
+                    dma::DataSize::S16,
+                    Default::default(),
+                );
+            }
+
+            /// Enable a specific type of ADC interrupt.
+            pub fn enable_interrupt(&mut self, interrupt: AdcInterrupt) {
+                match interrupt {
+                    AdcInterrupt::Ready => self.regs.ier.modify(|_, w| w.adrdyie().set_bit()),
+                    AdcInterrupt::EndOfConversion => self.regs.ier.modify(|_, w| w.eocie().set_bit()),
+                    AdcInterrupt::EnfOfSequence => self.regs.ier.modify(|_, w| w.eosie().set_bit()),
+                    AdcInterrupt::EndofConversionInjected => self.regs.ier.modify(|_, w| w.jeocie().set_bit()),
+                    AdcInterrupt::EndOfSequenceInjected => self.regs.ier.modify(|_, w| w.jeosie().set_bit()),
+                    AdcInterrupt::Watchdog1 => self.regs.ier.modify(|_, w| w.awd1ie().set_bit()),
+                    AdcInterrupt::Watchdog2 => self.regs.ier.modify(|_, w| w.awd2ie().set_bit()),
+                    AdcInterrupt::Watchdog3 => self.regs.ier.modify(|_, w| w.awd3ie().set_bit()),
+                    AdcInterrupt::EndOfSamplingPhase => self.regs.ier.modify(|_, w| w.eosmpie().set_bit()),
+                    AdcInterrupt::Overrun => self.regs.ier.modify(|_, w| w.ovrie().set_bit()),
+                    AdcInterrupt::InjectedOverflow => self.regs.ier.modify(|_, w| w.jqovfie().set_bit()),
+                }
+            }
+
+            /// Clear an interrupt flag of the specified type. Consider running this in the
+            /// corresponding ISR.
+            pub fn clear_interrupt(&mut self, interrupt: AdcInterrupt) {
+                match interrupt {
+                    AdcInterrupt::Ready => self.regs.isr.modify(|_, w| w.adrdy().set_bit()),
+                    AdcInterrupt::EndOfConversion => self.regs.isr.modify(|_, w| w.eoc().set_bit()),
+                    AdcInterrupt::EnfOfSequence => self.regs.isr.modify(|_, w| w.eos().set_bit()),
+                    AdcInterrupt::EndofConversionInjected => self.regs.isr.modify(|_, w| w.jeoc().set_bit()),
+                    AdcInterrupt::EndOfSequenceInjected => self.regs.isr.modify(|_, w| w.jeos().set_bit()),
+                    AdcInterrupt::Watchdog1 => self.regs.isr.modify(|_, w| w.awd1().set_bit()),
+                    AdcInterrupt::Watchdog2 => self.regs.isr.modify(|_, w| w.awd2().set_bit()),
+                    AdcInterrupt::Watchdog3 => self.regs.isr.modify(|_, w| w.awd3().set_bit()),
+                    AdcInterrupt::EndOfSamplingPhase => self.regs.isr.modify(|_, w| w.eosmp().set_bit()),
+                    AdcInterrupt::Overrun => self.regs.isr.modify(|_, w| w.ovr().set_bit()),
+                    AdcInterrupt::InjectedOverflow => self.regs.isr.modify(|_, w| w.jqovf().set_bit()),
+                }
+                // match interrupt {
+                //     AdcInterrupt::Ready => self.regs.icr.write(|_w| w.adrdy().set_bit()),
+                //     AdcInterrupt::EndOfConversion => self.regs.icr.write(|w| w.eoc().set_bit()),
+                //     AdcInterrupt::EnfOfSequence => self.regs.icr.write(|_w| w.eos().set_bit()),
+                //     AdcInterrupt::EndofConversionInjected => self.regs.icr.write(|_w| w.jeoc().set_bit()),
+                //     AdcInterrupt::EndOfSequenceInjected => self.regs.icr.write(|_w| w.jeos().set_bit()),
+                //     AdcInterrupt::Watchdog1 => self.regs.icr.write(|_w| w.awd1().set_bit()),
+                //     AdcInterrupt::Watchdog2 => self.regs.icr.write(|_w| w.awd2().set_bit()),
+                //     AdcInterrupt::Watchdog3 => self.regs.icr.write(|_w| w.awd3().set_bit()),
+                //     AdcInterrupt::EndOfSamplingPhase => self.regs.icr.write(|_w| w.eosmp().set_bit()),
+                //     AdcInterrupt::Overrun => self.regs.icr.write(|_w| w.ovr().set_bit()),
+                //     AdcInterrupt::InjectedOverflow => self.regs.icr.write(|_w| w.jqovf().set_bit()),
+                // }
+            }
         }
 
         impl<WORD, PIN> OneShot<pac::$ADC, WORD, PIN> for Adc<pac::$ADC>
@@ -757,17 +945,17 @@ hal!(ADC4, ADC3_4, adc4, 34);
 #[cfg(any(feature = "l4"))]
 hal!(ADC1, ADC_COMMON, adc1, _);
 
-// todo: ADC 1 vs 2 on L5? L5 supports up to 2 ADCs, so I'm not sure what's going on here.
-#[cfg(any(feature = "l5"))]
-hal!(ADC, ADC_COMMON, adc1, _);
-
 #[cfg(any(feature = "l4x1", feature = "l4x2", feature = "l4x5", feature = "l4x6",))]
 hal!(ADC2, ADC_COMMON, adc2, _);
 
 #[cfg(any(feature = "l4x5", feature = "l4x6",))]
 hal!(ADC3, ADC_COMMON, adc3, _);
 
-// todo Implement ADC3 on H7.
+// todo: ADC 1 vs 2 on L5? L5 supports up to 2 ADCs, so I'm not sure what's going on here.
+#[cfg(any(feature = "l5"))]
+hal!(ADC, ADC_COMMON, adc1, _);
+
+// todo Implement ADC3 on H7. The issue is the enable / reset being on ahb4.
 cfg_if! {
     if #[cfg(feature = "h7")] {
         hal!(ADC1, ADC12_COMMON, adc1, 12);
@@ -776,6 +964,8 @@ cfg_if! {
     }
 }
 
+#[cfg(feature = "h7")]
+
 cfg_if! {
     if #[cfg(feature = "g4")] {
         hal!(ADC1, ADC12_COMMON, adc1, 12);
@@ -783,16 +973,14 @@ cfg_if! {
     }
 }
 
-// todo: Implement ADCs 3, 4, 5 on G4 variants that support it.
-// cfg_if! {
-//     if #[cfg(any(feature = "g4..."))] {
-//         hal!(ADC3, ADC345_COMMON, adc3, 345);
-//         hal!(ADC4, ADC345_COMMON, adc4, 345);
-//         hal!(ADC5, ADC345_COMMON, adc5, 345);
-//     }
-// }
+#[cfg(all(feature = "g4", not(any(feature = "g431", feature = "g441"))))]
+hal!(ADC3, ADC345_COMMON, adc3, 345);
 
-// #[cfg(all(feature = "f4")]
-// hal!(ADC1, ADC1, adc1, 1);
+cfg_if! {
+    if #[cfg(any(feature = "g473", feature = "g474", feature = "g483", feature = "g484"))] {
+        hal!(ADC4, ADC345_COMMON, adc4, 345);
+        hal!(ADC5, ADC345_COMMON, adc5, 345);
+    }
+}
 
 // todo F4 as (depending on variant?) ADC 1, 2, 3
