@@ -2,13 +2,16 @@
 
 use crate::{
     clocks::SpeedError,
-    pac::{FLASH, RCC},
+    pac::{self, FLASH, RCC},
     rcc_en_reset,
     traits::{ClockCfg, ClocksValid},
 };
 
 use cfg_if::cfg_if;
 
+// todo: MSI and HSI calibration with LSE. The L4 RM describes this as possible, but doesn't tell you how
+// todo to do it. Something about tim15 and tim16? `RCC_ICSCR` resgister?
+
 #[cfg(not(feature = "g0"))]
 #[derive(Clone, Copy, PartialEq)]
 #[repr(u8)]
@@ -325,7 +328,7 @@ impl Pllr {
 #[cfg(not(feature = "g0"))]
 #[derive(Clone, Copy)]
 #[repr(u8)]
-// Main PLL division factor for PLLCLK (system clock). G4 RM 7.4.4
+// Main PLL division factor for PLLCLK (system clock). G4 RM 7.4.4. Also used to set PLLQ.
 pub enum Pllr {
     Div2 = 0b00,
     Div4 = 0b01,
@@ -410,6 +413,7 @@ pub struct Clocks {
     #[cfg(not(any(feature = "g0", feature = "g4")))]
     pub pll_sai2_mul: u8, // PLL SAI2 multiplier. Valid range of 7 to 86.
     pub pllr: Pllr,
+    pub pllq: Pllr,
     /// The value to divide SYSCLK by, to get systick and peripheral clocks. Also known as AHB divider
     pub hclk_prescaler: HclkPrescaler,
     /// The divider of HCLK to get the APB1 peripheral clock
@@ -452,7 +456,7 @@ impl Clocks {
 
         // Adjust flash wait states according to the HCLK frequency.
         // We need to do this before enabling PLL, or it won't enable.
-        let (_, sysclk) = calc_sysclock(self.input_src, self.pllm, self.plln, self.pllr);
+        let (_, sysclk) = self.calc_sysclock();
 
         let hclk = sysclk / self.hclk_prescaler.value() as u32;
 
@@ -620,14 +624,16 @@ impl Clocks {
                         w.pllsrc().bits(pll_src.bits());
                         w.plln().bits(self.plln);
                         w.pllm().bits(self.pllm as u8);
-                        w.pllr().bits(self.pllr as u8)
+                        w.pllr().bits(self.pllr as u8);
+                        w.pllq().bits(self.pllq as u8)
                     });
                 } else {
                     rcc.pllcfgr.modify(|_, w| unsafe {
                         w.pllsrc().bits(pll_src.bits());
                         w.plln().bits(self.plln);
                         w.pllm().bits(self.pllm as u8);
-                        w.pllr().bits(self.pllr as u8)
+                        w.pllr().bits(self.pllr as u8);
+                        w.pllq().bits(self.pllq as u8)
                     });
                 }
             }
@@ -782,7 +788,7 @@ impl Clocks {
 
     /// Re-select input source; used after Stop and Standby modes, where the system reverts
     /// to MSI or HSI after wake.
-    pub(crate) fn re_select_input(&self, rcc: &mut RCC) {
+    pub fn reselect_input(&self, rcc: &mut RCC) {
         // Re-select the input source; useful for changing input source, or reverting
         // from stop or standby mode. This assumes we're on a clean init,
         // or waking up from stop mode etc.
@@ -812,9 +818,14 @@ impl Clocks {
                         // If on G, we'll already be on HSI, so need to take action.
                     }
                     #[cfg(not(any(feature = "g0", feature = "g4")))]
-                    PllSrc::Msi(_) => {
+                    PllSrc::Msi(range) => {
+                        // Range initializes to 4Mhz, so set that as well.
+                        rcc.cr.modify(|_, w| unsafe {
+                            w.msirange().bits(range as u8).msirgsel().set_bit()
+                        });
                         if let StopWuck::Hsi = self.stop_wuck {
                             rcc.cr.modify(|_, w| w.msion().set_bit());
+
                             while rcc.cr.read().msirdy().bit_is_clear() {}
                         }
                     }
@@ -852,7 +863,11 @@ impl Clocks {
                 }
             }
             #[cfg(not(any(feature = "g0", feature = "g4")))]
-            InputSrc::Msi(_) => {
+            InputSrc::Msi(range) => {
+                // Range initializes to 4Mhz, so set that as well.
+                rcc.cr
+                    .modify(|_, w| unsafe { w.msirange().bits(range as u8).msirgsel().set_bit() });
+
                 if let StopWuck::Hsi = self.stop_wuck {
                     rcc.cr.modify(|_, w| w.msion().set_bit());
                     while rcc.cr.read().msirdy().bit_is_clear() {}
@@ -902,9 +917,11 @@ impl Clocks {
     }
 
     #[cfg(any(feature = "l4", feature = "l5"))]
-    /// Enables MSI, and configures it at 48Mhz. This is useful when using it as the USB clock,
-    /// ie with `clk48_src: Clk48Src::Msi`. Don't use this if using MSI for the input source or PLL source.
-    /// You may need to re-run this after exiting `stop` mode.
+    /// Enables MSI, and configures it at 48Mhz, and trims it using the LSE. This is useful when using it as
+    /// the USB clock, ie with `clk48_src: Clk48Src::Msi`. Don't use this if using MSI for the input
+    /// source or PLL source. You may need to re-run this after exiting `stop` mode. Only works for USB
+    /// if you have an LSE connected.
+    /// Note: MSIPLLEN must be enabled after LSE is enabled. So, run this function after RCC clock setup.
     pub fn enable_msi_48(&self, rcc: &mut RCC) {
         if let InputSrc::Msi(_) = self.input_src {
             panic!(
@@ -919,33 +936,91 @@ impl Clocks {
                 if not using it as the input source."
                 );
             }
-            panic!(
-                "Only use this function to set up MSI as 48MHz oscillator \
-            if not using it as the input source."
-            );
         }
 
-        // todo: Calibrate MSI with LSE / HSE(?) if avail?
         rcc.cr.modify(|_, w| w.msion().clear_bit());
         while rcc.cr.read().msirdy().bit_is_set() {}
 
+        // L44 RM, section 6.2.3: When a 32.768 kHz external oscillator is present in the application, it is possible to configure
+        // the MSI in a PLL-mode by setting the MSIPLLEN bit in the Clock control register (RCC_CR).
+        // When configured in PLL-mode, the MSI automatically calibrates itself thanks to the LSE.
+        // This mode is available for all MSI frequency ranges. At 48 MHz, the MSI in PLL-mode can
+        // be used for the USB FS device, saving the need of an external high-speed crystal.
+        // MSIPLLEN must be enabled after LSE is enabled
         rcc.cr.modify(|_, w| unsafe {
             w.msirange()
                 .bits(MsiRange::R48M as u8)
                 .msirgsel()
                 .set_bit()
+                .msipllen()
+                .set_bit()
                 .msion()
                 .set_bit()
         });
 
         while rcc.cr.read().msirdy().bit_is_clear() {}
     }
+
+    /// Calculate the systick, and input frequency, in Hz.
+    fn calc_sysclock(&self) -> (u32, u32) {
+        let input_freq;
+        let sysclk = match self.input_src {
+            InputSrc::Pll(pll_src) => {
+                input_freq = match pll_src {
+                    #[cfg(not(any(feature = "g0", feature = "g4")))]
+                    PllSrc::Msi(range) => range.value() as u32,
+                    PllSrc::Hsi => 16_000_000,
+                    PllSrc::Hse(freq) => freq,
+                    PllSrc::None => 0, // todo?
+                };
+                input_freq / self.pllm.value() as u32 * self.plln as u32 / self.pllr.value() as u32
+            }
+
+            #[cfg(not(any(feature = "g0", feature = "g4")))]
+            InputSrc::Msi(range) => {
+                input_freq = range.value() as u32;
+                input_freq
+            }
+            InputSrc::Hsi => {
+                input_freq = 16_000_000;
+                input_freq
+            }
+            InputSrc::Hse(freq) => {
+                input_freq = freq;
+                input_freq
+            }
+            #[cfg(feature = "g0")]
+            InputSrc::Lsi => {
+                input_freq = 32_000; // todo confirm this is right.
+                input_freq
+            }
+            #[cfg(feature = "g0")]
+            InputSrc::Lse => {
+                input_freq = 32_768;
+                input_freq
+            }
+        };
+
+        (input_freq, sysclk)
+    }
+
+    /// Check if the PLL is enabled. This is useful if checking wheather to re-enable the PLL
+    /// after exiting Stop or Standby modes, eg so you don't re-enable if it was already re-enabled
+    /// in a different context. eg:
+    /// ```
+    /// if !clock_cfg.pll_is_enabled() {
+    ///     clock_cfg.reselect_input(&mut dp.RCC);
+    ///}
+    ///```
+    pub fn pll_is_enabled(&self, rcc: &mut RCC) -> bool {
+        rcc.cr.read().pllon().bit_is_set()
+    }
 }
 
 // todo: Some extra calculations here, vice doing it once and caching.
 impl ClockCfg for Clocks {
     fn sysclk(&self) -> u32 {
-        let (_, sysclk) = calc_sysclock(self.input_src, self.pllm, self.plln, self.pllr);
+        let (_, sysclk) = self.calc_sysclock();
         sysclk
     }
 
@@ -1100,6 +1175,7 @@ impl Default for Clocks {
             #[cfg(not(any(feature = "g0", feature = "g4")))]
             pll_sai2_mul: 8,
             pllr: Pllr::Div2,
+            pllq: Pllr::Div4,
             hclk_prescaler: HclkPrescaler::Div1,
             apb1_prescaler: ApbPrescaler::Div1,
             #[cfg(not(feature = "g0"))]
@@ -1122,45 +1198,20 @@ impl Default for Clocks {
     }
 }
 
-/// Calculate the systick, and input frequency, in Hz.
-fn calc_sysclock(input_src: InputSrc, pllm: Pllm, plln: u8, pllr: Pllr) -> (u32, u32) {
-    let input_freq;
-    let sysclk = match input_src {
-        InputSrc::Pll(pll_src) => {
-            input_freq = match pll_src {
-                #[cfg(not(any(feature = "g0", feature = "g4")))]
-                PllSrc::Msi(range) => range.value() as u32,
-                PllSrc::Hsi => 16_000_000,
-                PllSrc::Hse(freq) => freq,
-                PllSrc::None => 0, // todo?
-            };
-            input_freq / pllm.value() as u32 * plln as u32 / pllr.value() as u32
-        }
-
-        #[cfg(not(any(feature = "g0", feature = "g4")))]
-        InputSrc::Msi(range) => {
-            input_freq = range.value() as u32;
-            input_freq
-        }
-        InputSrc::Hsi => {
-            input_freq = 16_000_000;
-            input_freq
-        }
-        InputSrc::Hse(freq) => {
-            input_freq = freq;
-            input_freq
-        }
-        #[cfg(feature = "g0")]
-        InputSrc::Lsi => {
-            input_freq = 32_000; // todo confirm this is right.
-            input_freq
-        }
-        #[cfg(feature = "g0")]
-        InputSrc::Lse => {
-            input_freq = 32_768;
-            input_freq
-        }
-    };
-
-    (input_freq, sysclk)
+#[cfg(any(feature = "l4", feature = "l5"))]
+/// Enable the Clock Recovery System. L443 User manual:
+/// "The STM32L443xx devices embed a special block which allows automatic trimming of the
+/// internal 48 MHz oscillator to guarantee its optimal accuracy over the whole device
+/// operational range. This automatic trimming is based on the external synchronization signal,
+/// which could be either derived from USB SOF signalization, from LSE oscillator, from an
+/// external signal on CRS_SYNC pin or generated by user software. For faster lock-in during
+/// startup it is also possible to combine automatic trimming with manual trimming action."
+/// Note: This is for HSI48 only.
+pub fn enable_crs(crs: &mut pac::CRS, rcc: &mut RCC) {
+    rcc.apb1enr1.modify(|_, w| w.crsen().set_bit());
+    // Initialize clock recovery
+    // Set autotrim enabled.
+    crs.cr.modify(|_, w| w.autotrimen().set_bit());
+    // Enable CR
+    crs.cr.modify(|_, w| w.cen().set_bit());
 }
