@@ -23,11 +23,11 @@
 //! which all 8 IOs are used for the interface).
 
 use crate::{
-    pac::{self, QUADSPI, RCC},
+    pac::{QUADSPI, RCC},
     traits::ClockCfg,
 };
 
-use core::{marker::PhantomData, ptr};
+use core::ptr;
 
 use cfg_if::cfg_if;
 
