@@ -79,18 +79,18 @@ pub struct Rtc {
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub struct RtcConfig {
     /// RTC clock source
-    clock_source: RtcClockSource,
+    pub clock_source: RtcClockSource,
     /// Asynchronous prescaler factor
     /// This is the asynchronous division factor:
     /// ck_apre frequency = RTCCLK frequency/(PREDIV_A+1)
     /// ck_apre drives the subsecond register
-    async_prescaler: u8,
+    pub async_prescaler: u8,
     /// Synchronous prescaler factor
     /// This is the synchronous division factor:
     /// ck_spre frequency = ck_apre frequency/(PREDIV_S+1)
     /// ck_spre must be 1Hz
-    sync_prescaler: u16,
-    bypass_lse_output: bool,
+    pub sync_prescaler: u16,
+    pub bypass_lse_output: bool,
 }
 
 impl Default for RtcConfig {
@@ -106,33 +106,6 @@ impl Default for RtcConfig {
     }
 }
 
-impl RtcConfig {
-    /// Sets the clock source of RTC config
-    pub fn clock_source(mut self, source: RtcClockSource) -> Self {
-        self.clock_source = source;
-        self
-    }
-
-    /// Set the asynchronous prescaler of RTC config
-    pub fn async_prescaler(mut self, prescaler: u8) -> Self {
-        self.async_prescaler = prescaler;
-        self
-    }
-
-    /// Set the synchronous prescaler of RTC config
-    pub fn sync_prescaler(mut self, prescaler: u16) -> Self {
-        self.sync_prescaler = prescaler;
-        self
-    }
-
-    /// Choose wheather to bypass the output line to the LSE, and configure
-    /// it as a GPIO
-    pub fn bypass_lse_output(mut self, bypass: bool) -> Self {
-        self.bypass_lse_output = bypass;
-        self
-    }
-}
-
 impl Rtc {
     /// Create and enable a new RTC abstraction, and configure its clock source and prescalers.
     /// From AN4759, Table 7, when using the LSE (The only clock source this module
@@ -140,8 +113,6 @@ impl Rtc {
     /// calendar clock of 1Hz.
     /// The `bypass` argument is `true` if you're using an external oscillator that
     /// doesn't connect to `OSC32_IN`, such as a MEMS resonator.
-    /// Note: You may need to run `dp.RCC.apb1enr.modify(|_, w| w.pwren().set_bit());` before
-    /// constraining RCC, eg before running this constructor.
     /// Note that if using HSE as the clock source, we assume you've already enabled it, eg
     /// in clock config.
     pub fn new(regs: RTC, rcc: &mut RCC, pwr: &mut PWR, config: RtcConfig) -> Self {
@@ -268,6 +239,7 @@ impl Rtc {
     // /// Setup the alarm. See AN4759, section 2.3.1.
     // /// `sleep_time` is in ms. `Table 8` desribes these steps.
     // pub fn set_alarm(&mut self, exti: &mut EXTI) {
+    // note: STM3241x and 42x have diff addresses, and are PAC incompatible!
     //     exti.imr1.modify(|_, w| w.mr18().unmasked());
     //     exti.rtsr1.modify(|_, w| w.tr18().bit(true));
     //     exti.ftsr1.modify(|_, w| w.tr18().bit(false));
