@@ -6,13 +6,20 @@
 
 // todo: Missing some features (like additional interrupts) on the USARTv3 peripheral . (L5, G etc)
 
-use core::{ops::Deref, ptr};
-
 use crate::{
     pac::{self, RCC},
     rcc_en_reset,
     traits::ClockCfg,
 };
+use core::ops::Deref;
+
+#[cfg(feature = "g0")]
+use crate::pac::dma as dma_p;
+#[cfg(any(feature = "f3", feature = "l4", feature = "g4"))]
+use crate::pac::dma1 as dma_p;
+
+#[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
+use crate::dma::{self, Dma, DmaChannel, DmaInput};
 
 use embedded_hal::{
     blocking,
@@ -220,9 +227,9 @@ where
             .modify(|_, w| unsafe { w.stop().bits(result.config.stop_bits as u8) });
         // 4. Enable the USART by writing the UE bit in USART_CR1 register to 1.
         result.regs.cr1.modify(|_, w| w.ue().set_bit());
-        // 5. Select DMA enable (DMAT) in USART_CR3 if multibuffer communication is to take
+        // 5. Select DMA enable (DMAT[R]] in USART_CR3 if multibuffer communication is to take
         // place. Configure the DMA register as explained in multibuffer communication.
-        // todo?
+        // (Handled in `enable_dma()`)
         // 6. Set the TE bit in USART_CR1 to send an idle frame as first transmission.
         // 6. Set the RE bit USART_CR1. This enables the receiver which begins searching for a
         // start bit.
@@ -339,19 +346,214 @@ where
         // reception of the next character to avoid an overrun error
     }
 
-    // /// Flush the transmit buffer.
-    // pub fn flush(&mut self) {
-    //
-    // }
+    /// Read a single word, without confirming if it's ready. This is useful in async concepts,
+    /// when you know word is ready to be read.
+    pub fn read_one(&mut self) -> u8 {
+        cfg_if! {
+            if #[cfg(not(feature = "f4"))] {
+                self.regs.rdr.read().rdr().bits() as u8
+            } else {
+                 self.regs.dr.read().dr().bits() as u8
+            }
+        }
+    }
+
+    #[cfg(any(feature = "f3", feature = "l4"))]
+    /// Enable use of DMA transmission for U[s]ART: (L44 RM, section 38.5.15)
+    pub fn enable_dma<D>(&mut self, dma: &mut Dma<D>)
+    where
+        D: Deref<Target = dma_p::RegisterBlock>,
+    {
+        // "DMA mode can be enabled for transmission by setting DMAT bit in the USART_CR3
+        // register. Data is loaded from a SRAM area configured using the DMA peripheral (refer to
+        // Section 11: Direct memory access controller (DMA) on page 295) to the USART_TDR
+        // register whenever the TXE bit is set."
+        self.regs.cr3.modify(|_, w| w.dmat().set_bit());
+        self.regs.cr3.modify(|_, w| w.dmar().set_bit());
+
+        // Note that we need neither channel select, nor multiplex for F3.
+    }
+
+    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    /// Transmit data using DMA. (L44 RM, section 38.5.15)
+    /// Note that the `channel` argument is only used on F3 and L4.
+    pub fn write_dma<D>(&mut self, buf: &[u8], channel: DmaChannel, dma: &mut Dma<D>)
+    where
+        D: Deref<Target = dma_p::RegisterBlock>,
+    {
+        let (ptr, len) = (buf.as_ptr(), buf.len());
+
+        // To map a DMA channel for USART transmission, use
+        // the following procedure (x denotes the channel number):
+
+        // todo: This channel selection is confirmed for L4 only - check other families
+        // todo DMA channel mapping
+        #[cfg(any(feature = "f3", feature = "l4"))]
+        let channel = match self.device {
+            UsartDevice::One => DmaInput::Usart1Tx.dma1_channel(),
+            UsartDevice::Two => DmaInput::Usart2Tx.dma1_channel(),
+            #[cfg(not(any(
+                feature = "f401",
+                feature = "f410",
+                feature = "f411",
+                feature = "f412",
+                feature = "f413",
+                feature = "l4x1",
+                feature = "g0"
+            )))]
+            UsartDevice::Three => DmaInput::Usart3Tx.dma1_channel(),
+        };
+
+        // todo: These are only valid for DMA1!
+        #[cfg(feature = "l4")]
+        match self.device {
+            UsartDevice::One => dma.channel_select(DmaInput::Usart1Tx),
+            UsartDevice::Two => dma.channel_select(DmaInput::Usart2Tx),
+            #[cfg(not(any(
+                feature = "f401",
+                feature = "f410",
+                feature = "f411",
+                feature = "f412",
+                feature = "f413",
+                feature = "l4x1",
+                feature = "g0"
+            )))]
+            UsartDevice::Three => dma.channel_select(DmaInput::Usart3Tx),
+        }
+
+        // todo: Pri and Circular as args?
+
+        dma.cfg_channel(
+            channel,
+            // 1. Write the USART_TDR register address in the DMA control register to configure it as
+            // the destination of the transfer. The data is moved to this address from memory after
+            // each TXE event.
+            &self.regs.tdr as *const _ as u32,
+            // 2. Write the memory address in the DMA control register to configure it as the source of
+            // the transfer. The data is loaded into the USART_TDR register from this memory area
+            // after each TXE event.
+            ptr as u32,
+            // 3. Configure the total number of bytes to be transferred to the DMA control register.
+            len as u16,
+            dma::Direction::ReadFromMem,
+            // 4. Configure the channel priority in the DMA control register
+            // (Handled by `ChannelCfg::default())`
+            dma::DataSize::S8,
+            dma::DataSize::S8,
+            Default::default(),
+        );
+
+        // 5. Configure DMA interrupt generation after half/ full transfer as required by the
+        // application.
+        // (Handled in `cfg_channel`)
+
+        // 6. Clear the TC flag in the USART_ISR register by setting the TCCF bit in the
+        // USART_ICR register.
+        self.regs.icr.write(|w| w.tccf().set_bit());
+
+        // 7. Activate the channel in the DMA register.
+        // When the number of data transfers programmed in the DMA Controller is reached, the DMA
+        // controller generates an interrupt on the DMA channel interrupt vector.
+        // (Handled in `cfg_channel`)
+
+        // In transmission mode, once the DMA has written all the data to be transmitted (the TCIF flag
+        // is set in the DMA_ISR register), the TC flag can be monitored to make sure that the USART
+        // communication is complete. This is required to avoid corrupting the last transmission before
+        // disabling the USART or entering Stop mode. Software must wait until TC=1. The TC flag
+        // remains cleared during all data transfers and it is set by hardware at the end of transmission
+        // of the last frame.
+    }
+
+    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    /// Receive data using DMA. (L44 RM, section 38.5.15)
+    /// Note that the `channel` argument is only used on F3 and L4.
+    pub fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
+    where
+        D: Deref<Target = dma_p::RegisterBlock>,
+    {
+        let (ptr, len) = (buf.as_mut_ptr(), buf.len());
+
+        #[cfg(any(feature = "f3", feature = "l4"))]
+        let channel = match self.device {
+            UsartDevice::One => DmaInput::Usart1Rx.dma1_channel(),
+            UsartDevice::Two => DmaInput::Usart2Rx.dma1_channel(),
+            #[cfg(not(any(
+                feature = "f401",
+                feature = "f410",
+                feature = "f411",
+                feature = "f412",
+                feature = "f413",
+                feature = "l4x1",
+                feature = "g0"
+            )))]
+            UsartDevice::Three => DmaInput::Usart3Rx.dma1_channel(),
+        };
+
+        #[cfg(feature = "l4")]
+        match self.device {
+            UsartDevice::One => dma.channel_select(DmaInput::Usart1Rx),
+            UsartDevice::Two => dma.channel_select(DmaInput::Usart2Rx),
+            #[cfg(not(any(
+                feature = "f401",
+                feature = "f410",
+                feature = "f411",
+                feature = "f412",
+                feature = "f413",
+                feature = "l4x1",
+                feature = "g0"
+            )))]
+            UsartDevice::Three => dma.channel_select(DmaInput::Usart3Rx),
+        }
+
+        // todo: Pri and Circular as args?
+
+        dma.cfg_channel(
+            channel,
+            // 1. Write the USART_RDR register address in the DMA control register to configure it as
+            // the source of the transfer. The data is moved from this address to the memory after
+            // each RXNE event.
+            &self.regs.tdr as *const _ as u32,
+            // 2. Write the memory address in the DMA control register to configure it as the destination
+            // of the transfer. The data is loaded from USART_RDR to this memory area after each
+            // RXNE event.
+            ptr as u32,
+            // 3. Configure the total number of bytes to be transferred to the DMA control register.
+            len as u16,
+            dma::Direction::ReadFromPeriph,
+            dma::DataSize::S8,
+            dma::DataSize::S8,
+            Default::default(),
+        );
+
+        // 5. Configure interrupt generation after half/ full transfer as required by the application.
+        // todo (Let the user call `dma.setup_interrupt()`? But that requires they know
+        // todo which channel of DMA usart tx is on.
+
+        // 6. Activate the channel in the DMA control register.
+        // When the number of data transfers programmed in the DMA Controller is reached, the DMA
+        // controller generates an interrupt on the DMA channel interrupt vector.
+        // (Handled in above fn call)
+
+        // When the number of data transfers programmed in the DMA Controller is reached, the DMA
+        // controller generates an interrupt on the DMA channel interrupt vector.
+    }
+
+    /// Flush the transmit buffer.
+    pub fn flush(&self) {
+        #[cfg(not(feature = "f4"))]
+        while self.regs.isr.read().tc().bit_is_clear() {}
+        #[cfg(feature = "f4")]
+        while self.regs.sr.read().tc().bit_is_clear() {}
+    }
 
     #[cfg(not(feature = "f4"))]
     /// Enable a specific type of interrupt.
-    pub fn enable_interrupt(&mut self, interrupt_type: UsartInterrupt) {
+    pub fn enable_interrupt(&mut self, interrupt: UsartInterrupt) {
         // Disable the UART to allow writing the `add` and `addm7` bits
         self.regs.cr1.modify(|_, w| w.ue().clear_bit());
         while self.regs.cr1.read().ue().bit_is_set() {}
 
-        match interrupt_type {
+        match interrupt {
             UsartInterrupt::CharDetect(char) => {
                 // Enable character-detecting UART interrupt
                 self.regs.cr1.modify(|_, w| w.cmie().set_bit());
@@ -414,8 +616,8 @@ where
 
     #[cfg(not(feature = "f4"))]
     /// Clears the interrupt pending flag for a specific type of interrupt.
-    pub fn clear_interrupt(&mut self, interrupt_type: UsartInterrupt) {
-        match interrupt_type {
+    pub fn clear_interrupt(&mut self, interrupt: UsartInterrupt) {
+        match interrupt {
             UsartInterrupt::CharDetect(_) => self.regs.icr.write(|w| w.cmcf().set_bit()),
             UsartInterrupt::Cts => self.regs.icr.write(|w| w.ctscf().set_bit()),
             UsartInterrupt::EndOfBlock => self.regs.icr.write(|w| w.eobcf().set_bit()),
